

# an OBJECT is a string like
# "INDEX:<TAGNAME [options...]> text </TAGNAME>"
# options are always like OPTION="xxx"
#

#### CheckTag ####
# performs a consistency check

sub CheckTag {
    local($tag)=@_;
    local($op,$pos,$name,$text,%opt);
    local($s)=0;

    $tag =~ tr/a-z/A-Z/;

    if($tag =~ m/^(\d+)\:<([^>]*)>/) {
	$pos = $1;
	$name = $2;
	$tag = $';

	$name =~ m/^(\S+)/;
	$name = $1;
	$op = $';

	$op =~ s/(\S+)\=\"([^\"]*)\"/{$opt{$1}=$2}/ge;

	if($tag =~ m/<\/$name>$/) {
	    $text = $`;
	    $s = 1;
	}
    }
    
    return($s,$pos,$name,$text,%opt);
}

#### MakeTag ####
# returns a TAG object

sub MakeTag {
    local($tagname)=@_;

    if($tagname) {
	$tagname =~ m/^ *(\S+)/;
	$tagname = $1;
	$tagname =~ tr/a-z/A-Z/;
	return "0:<$tagname> </$tagname>";
    }

    return undef;
}

#### GetTagName ####
# returns the value of an option

sub GetTagName {
    local($tag,$opt)=@_;

    # consistency check
    local($s,$pos,$name,$text,%opt)=CheckTag($tag);

    if( $s == 0 ) {
	return "";
    }

    return $name;
}
#### GetTagOpt ####
# returns the value of an option

sub GetTagOpt {
    local($tag,$opt)=@_;
    local($optval);

    $opt =~ tr/a-z/A-Z/;

    # consistency check
    local($s,$pos,$name,$text,%optlist)=CheckTag($tag);

    if( $s ==1) {
	$optval=$optlist{$opt};
    }

    return $optval;
}

#### GetTagVal ####
# returns the text contained 

sub GetTagVal {
    local($tag)=@_;

    # consistency check    
    local($s,$pos,$name,$text,%opt)=CheckTag($tag);

    if( $s == 0) {
	$text="";
    }

    return $text;
}




local($extag) = "5:<tag opt1=\"caccola \" pignatta=\"zombie\">pinc <tag>pino</tag></tag>";


local($s,$pos,$name,$text,%opt)=CheckTag($extag);

print "s=$s\n";
print "pos=$pos\n";
print "name=$name\n";
print "text=>>>$text<<<\n";

foreach $k (keys %opt) {
    print "opt <$k> = <$opt{$k}>\n";
}

