
########################################################################
#  Perl Library to Parse input from HTTPD                              #
#  version 1.21                                                        #
########################################################################
# WebMail is copyright 1995 CRS4 - Luca Manunza/Gianluigi Zanetti.     #
# The information (of all forms) in these files is the intellectual    #
# property of the Centro di Ricerca, Sviluppo e Studi Superiori in     #
# Sardegna (known as CRS4 - Center for Advanced Studies, Research and  #
# Development in Sardinia).                                            #
# No guarantee whatsoever is provided by CRS4. No liability whatsoever #
# is accepted for any loss or damage of any kind resulting from any    #
# defect or inaccuracy in this information or code.                    #
#                                                                      #
# Author: Luca Manunza <bilbo@crs4.it>                                 #
#                                                                      #
# First version: 20-02-1995                                            #
# Last Update  : 29-03-1995                                            #
########################################################################


sub Call_Method_Is_Get{
  return ($ENV{'REQUEST_METHOD'} eq "GET");
}

# ParseCGI
# Reads data from GET or POST method and write
# place PATH_INFO informations into @PATHINFO
# place values into @CGI as "KEY=VALUE" pairs
# place values into %CGI as $CGI{KEY}=VALUE
# multiple selections are '\0' separated
# if ISMAP coords are presents they are placed as ISMAP_X , ISMAP_Y vars

sub ParseCGI {

  local(*CGI,*PATHINFO) = @_ if @_;
  local($i,$x,$y, $loc);
  local($ismap, $key, $value);

  # split PATHINFO and waste first entry
  @PATHINFO = split('/',$ENV{'PATH_INFO'});
  shift @PATHINFO;
  $PATHINFO = join('/',@PATHINFO);

  # search the ismapx and ismapy coords
  $ismap=$ENV{'QUERY_STRING'};

  if($ismap =~ m/^(\d+)\,(\d+)$/) {
      $x = $1; $y = $2;
      $CGI[0] = "ISMAP_X=$x";
      $CGI[1] = "ISMAP_Y=$y";
      $CGI{"ISMAP_X"}=$x;
      $CGI{"ISMAP_Y"}=$y;
  } else {

      # Read the CGI query text 
      if (&Call_Method_Is_Get()) {
	  $CGI = $ENV{'QUERY_STRING'};
      } else {			
	  read(STDIN,$CGI,$ENV{'CONTENT_LENGTH'});
      }

      # Split values on & char (HTTPD specs)
      @CGI = split('&',$CGI);
      
      # make the hash array
      foreach $i (0 .. $#CGI) {
	  # WWW browser send space as plus then I have to convert plus to space
	  $CGI[$i] =~ s/\+/ /g;
	  
	  # Split pairs into key and value on the first =
	  ($key, $val) = split('=',$CGI[$i],2);

	  # Convert %XX from hex numbers to alphanumeric chars
	  $key =~ s/%(..)/pack("c",hex($1))/ge;
	  $val =~ s/%(..)/pack("c",hex($1))/ge;

	  # Associate key and value
	  $CGI{$key} .= "\0" if (defined($CGI{$key})); # \0 is the multiple separator
	  $CGI{$key} .= $val;
	  
      }
  }				
}

# CGIVarList
# Build an HTML dotted list from %CGI array

sub CGIVarList {
  local (*CGI) = @_ if @_;
  local ($list) = "<UL>";
  local ($line,$key);

  #pattern match is multiline
  local ($old_multiline_status) = $*;  $* =1;
  
  $list .=  "<DL COMPACT>";
  foreach $key (keys(%CGI)) {	
      foreach $line (split("\0", $CGI{$key})) {
	  # replace newline with <BR>
	  $line =~ s/\n/<BR>/g;
	  $list .=  "<LI><B>$key</B> = <I>$line</I><BR>";
      }
  }				
  $list .=  "</DL>";		 
  $* = $old_multiline_status;

  $list;			# returns the string
}


1; #return true (for perl's happyness)






